<?php
/*
Plugin Name: VK.com Video Link Checker
Description: N/A
Version: 1.0
Author: Janis Elsts

ModuleID: vk-com-checker
ModuleCategory: checker
ModuleContext: on-demand
ModuleLazyInit: true
ModuleClassName: blcVkVideoChecker
ModulePriority: 100

ModuleCheckerUrlPattern: @^https?://(?:www\.)?vk\.com/video_ext\.@i
*/

/**
 * VK.com video checker.
 *
 * @package Broken Link Checker
 * @author Janis Elsts
 * @access public
 */
class blcVkVideoChecker extends blcChecker {

	/**
	 * Determine if the checker can parse a specific URL.
	 * Always returns true because the ModuleCheckerUrlPattern header constitutes sufficient verification.
	 *
	 * @param string $url
	 * @param array $parsed
	 * @return bool True.
	 */
	function can_check($url, $parsed){
		return true;
	}

	/**
	 * Check an embedded video from vk.com.
	 *
	 * @param string $url Video URL.
	 * @return array
	 */
	function check($url){
		$result = array(
			'final_url' => $url,
			'redirect_count' => 0,
			'timeout' => false,
			'broken' => false,
			'log' => sprintf("<em>(%s)</em>\n\n", 'Using vk.com video checker'),
			'result_hash' => '',
			'status_code' => '',
			'status_text' => '',
		);

		$start = microtime_float();
		$response = wp_remote_get($url);
		$result['request_duration'] = microtime_float() - $start;

		if ( is_wp_error($response) ){
			$result['log'] .= "Error : " . $response->get_error_message();
			$result['broken'] = true;
			$result['http_code'] = 0;
		} else {
			$result['http_code'] = intval($response['response']['code']);

			if ( $result['http_code'] != 200 ) {
				$result['log'] .= "Error : Unexpected HTTP status code.";
				$result['broken'] = true;
			} else {
				$dom = new DOMDocument();
				@$dom->loadHTML($response['body']);

				//If the video URL is valid then the page will contain a video player.
				//The player is inside a <div> with the ID = "playerWrap".
				$playerDiv = $dom->getElementById('playerWrap');
				if ( !empty($playerDiv) ) {
					$result['status_code'] = BLC_LINK_STATUS_OK;
					$result['status_text'] = _x('OK', 'link status', 'broken-link-checker');
					$result['log'] .= "Video found.";
				} else {
					//No luck - it seems the video doesn't exist or access is restricted.
					$result['broken'] = true;
					$result['status_code'] = BLC_LINK_STATUS_ERROR;
					$result['status_text'] = __('Not Found', 'broken-link-checker');

					//There should be a #page_wrap node containing an error message.
					$errorDiv = $dom->getElementById('page_wrap');
					if ( !empty($errorDiv) ) {
						$message = trim($errorDiv->textContent);
						//We have no idea how long the error message might get (we're just screen-scraping,
						//after all), so lets limit it to something reasonable.
						$message = substr($message, 0, 200);
						if ( !empty($message) ) {
							$result['log'] .= "Error message: " . $message;
						}
					}
				}
			}
		}

		//Generate the result hash (used for detecting false positives)
		$result['result_hash'] = implode('|', array(
			'rapidshare',
			$result['http_code'],
			$result['broken'] ? 'broken' : '0',
			$result['timeout'] ? 'timeout' : '0'
		));

		return $result;
	}

}
